@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/bin/sh
 # t/testc.sh -c -Du,-q -B static 2>&1 |tee c.log|grep FAIL
 # for p in 5.6.2 5.8.8-nt 5.8.9d 5.10.1d 5.10.1d-nt 5.11.2d 5.11.2d-nt; do make -s clean; echo perl$p; perl$p Makefile.PL; t/testc.sh -q -O0 31; done
 # quiet c only: t/testc.sh -q -O0
@@ -6,6 +6,7 @@ function help {
   echo "t/testc.sh [OPTIONS] [1-$ntests]"
   echo " -D<debugflags>     for O=C or O=CC. Default: C,-DcOACMSGpu,-v resp. CC,-DoOscprSql,-v"
   echo " -O<0-4>            optimization level"
+  echo " -f<opt>            special optimization"
   echo " -B<static|dynamic> pass to cc_harness"
   echo " -c                 continue on errors"
   echo " -k                 keep temp. files on PASS"
@@ -30,30 +31,28 @@ Mblib=${Mblib:--Mblib} # B::C is now fully 5.6+5.8 backwards compatible
 if [ -z $Mblib ]; then 
     VERS="${VERS}_global"; 
     OCMD="$PERL $Mblib -MO=C,-DcAC,"
-    if [ $BASE = "testcc.sh" ]; then 
-        OCMD="$PERL $Mblib -MO=CC,-DrOsplt,"
+    if [ $BASE = "testcc.sh" ]; then # DrOsplt 
+        OCMD="$PERL $Mblib -MO=CC,-DOsplt,"
     fi
 else
     OCMD="$PERL $Mblib -MO=C,-DcoOSAHGCMpu,-v,"
-    if [ $BASE = "testcc.sh" ]; then
-        OCMD="$PERL $Mblib -MO=CC,-DoOscprSql,-v,"
+    if [ $BASE = "testcc.sh" ]; then # DoOscprSql
+        OCMD="$PERL $Mblib -MO=CC,-DoOscpSql,-v,"
     fi
 fi
-OCMDO1="$(echo $OCMD|sed -e s/C,-D/C,-O1,-D/)"
-OCMDO2="$(echo $OCMD|sed -e s/C,-D/C,-O2,-D/)"
-OCMDO3="$(echo $OCMD|sed -e s/C,-D/C,-O3,-D/)"
-OCMDO4="$(echo $OCMD|sed -e s/C,-D/C,-O4,-D/)"
 CONT=
 # 5.6: rather use -B static
 #CCMD="$PERL script/cc_harness -g3"
 # rest. -DALLOW_PERL_OPTIONS for -Dtlv
-CCMD="$PERL $Mblib script/cc_harness -d -g3 -DALLOW_PERL_OPTIONS"  
+#CCMD="$PERL $Mblib script/cc_harness -g3 -DALLOW_PERL_OPTIONS"
+CCMD="$PERL $Mblib script/cc_harness"
 LCMD=
 # On some perls I also had to add $archlib/DynaLoader/DynaLoader.a to libs in Config.pm
 }
 
 function vcmd {
     test -n "$QUIET" || echo $*
+    #echo $*
     $*
 }
 
@@ -75,17 +74,12 @@ function fail {
 function runopt {
     o=$1
     optim=$2
+    OCMDO1="$(echo $OCMD|sed -e s/C,/C,-O$optim,/)"
     suff="_o${optim}"
     if [ "$optim" == "0" ]; then suff=""; fi
     rm ${o}${suff} ${o}${suff}.c 2> /dev/null
-    if [ $optim == 1 ]; then CMD=$OCMDO1
-     else if [ $optim == 2 ]; then CMD=$OCMDO2
-      else if [ $optim == 3 ]; then CMD=$OCMDO3
-       else if [ $optim == 4 ]; then CMD=$OCMDO4
-        else CMD=$OCMD
-       fi
-      fi
-     fi
+    if [ $optim -lt 5 ]; then CMD=$OCMDO1
+    else CMD=$OCMD
     fi
     vcmd ${CMD}-o${o}${suff}.c $o.pl
     test -z $CPP || vcmd $CCMD ${o}${suff}.c -c -E -o ${o}${suff}_E.c
@@ -121,9 +115,12 @@ function ctest {
     else
 	echo "$str" > ${o}.pl
     fi
-    if [ $OPTIM -ge 0 ]; then
+    if [ -z "$str" ]; then
+      true
+    else
+      if [ $OPTIM -ge 0 ]; then
 	runopt "$o" "$OPTIM"
-    else # -1
+      else # -1
 	rm $o.c $o ${o}_o.c ${o}_o 2> /dev/null
 	vcmd ${OCMD}-o$o.c $o.pl
         test -s $o.c || (echo "empty $o.c"; test -z $CONT && exit)
@@ -146,13 +143,14 @@ function ctest {
 	    fail "./$o" "'$str' => '$res' Expected: '${result[$n]}'"
 	    test -z $CONT && exit
 	fi
+      fi
     fi
 }
 
-ntests=42
+ntests=49
 declare -a tests[$ntests]
 declare -a result[$ntests]
-ncctests=3
+ncctests=23
 declare -a cctests[$((100+$ncctests))]
 declare -a ccresult[$((100+$ncctests))]
 tests[1]='print "hi"'
@@ -222,18 +220,19 @@ tests[24]='sub level1{return(level2()?"fail":"ok")} sub level2{0} print level1()
 result[24]='ok'
 # enforce custom ncmp sort and count it. fails as CC in all. How to enforce icmp?
 # <=5.6 qsort needs two more passes here than >=5.8 merge_sort
+# 5.12 got it backwards and added 4 more passes.
 tests[25]='print sort { print $i++," "; $b <=> $a } 1..4'
-result[25]="0 1 2 3`$PERL -e'print (($] < 5.007) ? q( 4 5) : q())'` 4321";
+result[25]="0 1 2 3`$PERL -e'print (($] < 5.007) ? q( 4 5) : $] > 5.013 ? q( 4 5 6 7) : q())'` 4321";
 # lvalue sub
 tests[26]='sub a:lvalue{my $a=26; ${\(bless \$a)}}sub b:lvalue{${\shift}}; print ${a(b)}';
 result[26]="26";
-# xsub constants
-tests[27]='use Fcntl (); print "ok" if ( Fcntl::O_CREAT() == 64 && &Fcntl::O_CREAT == 64 ); '
+# xsub constants (constant folded). newlib: 0x200, glibc: 0x100
+tests[27]='use Fcntl ();my $a=Fcntl::O_CREAT(); print "ok" if ( $a >= 64 && &Fcntl::O_CREAT >= 64 );'
 result[27]='ok'
 # require $fname
-tests[28]='my($fname,$tmp_fh);while(!open($tmp_fh,">",($fname=q{cctest28_} . rand(999999999999)))){$bail++;die "Failed to create a tmp file after 500 tries" if $bail>500;}print {$tmp_fh} q{$x="ok";1;};close($tmp_fh);require $fname;unlink($fname);print $x;'
+tests[28]='my($fname,$tmp_fh);while(!open($tmp_fh,">",($fname=q{ccode28_} . rand(999999999999)))){$bail++;die "Failed to create a tmp file after 500 tries" if $bail>500;}print {$tmp_fh} q{$x="ok";1;};close($tmp_fh);sleep 1;require $fname;END{unlink($fname);};print $x;'
 result[28]='ok'
-# use test
+# special old IO handling
 tests[29]='use IO;print "ok"'
 result[29]='ok'
 # run-time context of .., fails in CC
@@ -250,12 +249,12 @@ result[32]='12'
 # fails with new constant only. still not repro (r-magic probably)
 tests[33]='BEGIN{unshift @INC,("t");} use qr_loaded_module; print "ok"'
 result[33]='ok'
-# init of magic hashes. %ENV has e magic since a0714e2c perl.c  
+# init of magic hashes. %ENV has e magic since a0714e2c perl.c
 # (Steven Schubiger      2006-02-03 17:24:49 +0100 3967) i.e. 5.8.9 but not 5.8.8
 tests[34]='my $x=$ENV{TMPDIR};print "ok"'
 result[34]='ok'
 # method_named. fixed with 1.16
-tests[35]='package dummy;sub meth{print "ok"};package main;dummy->meth'
+tests[35]='package dummy;sub meth{print "ok"};package main;dummy->meth(1)'
 result[35]='ok'
 # HV self-ref
 tests[36]='my ($rv, %hv); %hv = ( key => \$rv ); $rv = \%hv; print "ok";'
@@ -268,8 +267,7 @@ tests[38]='for(1 .. 1024) { if (open(my $null_fh,"<","/dev/null")) { seek($null_
 result[38]='ok'
 # check re::is_regexp, and on 5.12 if being upgraded to SVt_REGEXP
 # => Undefined subroutine &re::is_regexp with B-C-1.19, even with -ure
-usere="`$PERL -e'print (($] < 5.011) ? q(use re;) : q())'`"
-tests[39]='{'$usere'$a=${qr//};$a=2;print ($] < 5.007?1:re::is_regexp(\$a))}'
+tests[39]='{$a=qr/x/;print($]<5.010?1:re::is_regexp($a))}'
 result[39]='1'
 # String with a null byte -- used to generate broken .c on 5.6.2 with static pvs
 tests[40]='my $var="this string has a null \\000 byte in it";print "ok";'
@@ -290,9 +288,25 @@ result[43]='ok'
 # perl #72922 (5.11.4 fails with magic_killbackrefs)
 tests[44]='use Scalar::Util "weaken";my $re1=qr/foo/;my $re2=$re1;weaken($re2);print "ok" if $re3=qr/$re1/;'
 result[44]='ok'
-# test autoload and xs_init
-tests[45]='use Data::Dumper ();Data::Dumper::Dumper(\%main::);print "ok";'
+# test dynamic loading
+tests[45]='use Data::Dumper ();Data::Dumper::Dumpxs({});print "ok";'
 result[45]='ok'
+# Exporter should end up in main:: stash when used in
+tests[46]='use Exporter; if (exists $main::{"Exporter::"}) { print "ok"; }'
+result[46]='ok'
+# non-tied av->MAGICAL
+tests[47]='@ISA=(q(ok));print $ISA[0];'
+result[47]='ok'
+# m//i
+tests[48]='print q(ok) if "test" =~ /es/i;'
+result[48]='ok'
+#-------------
+# issue27
+tests[49]='require LWP::UserAgent;print q(ok);'
+result[49]='ok'
+# issue24
+tests[50]='dbmopen(%H,q(f),0644);print q(ok);'
+result[50]='ok'
 
 # from here on we test CC specifics only
 
@@ -305,16 +319,100 @@ result[102]='ok'
 # CC stringify, srefgen. TODO: use B; fails
 tests[103]='require B; my $x=1e1; my $s="$x"; print ref B::svref_2object(\$s)'
 result[103]='B::PV'
+# CC reset
+tests[104]='@a=(1..4);while($a=shift@a){print $a;}continue{$a=~/2/ and reset q(a);}'
+result[104]='12'
+# CC attrs. requires -MB::CC with pure perl
+tests[105]='use blib;use B::CC;my int $r;my $i:int=2;our double $d=3.0; $r=$i*$i; $r*=$d; print $r;'
+result[105]='12'
 
+# issue35
+tests[110]='sub new{}sub test{{my $x=1;my $y=$x+1;}my $x=2;if($x!=3){4;}}'
+result[110]=''
+# issue36
+tests[111]='sub f{shift==2}sub test{while(1){last if f(2);}while(1){last if f(2);}}'
+result[111]=''
+# issue37
+tests[112]='my $x;$x||=1;print "ok" if $x;'
+result[112]='ok'
+# issue38
+tests[113]='my $x=2;$x=$x||3;print "ok" if $x==2;'
+result[113]='ok'
+# issue39
+tests[114]='sub f1{0}sub f2{my $x;if(f1()){}if($x){}else{[$x]}}my @a=f2();print "ok";'
+result[114]='ok'
+# issue42
+tests[115]='sub f1{1}f1();print do{7;2},"\n";'
+result[115]='2'
+# issue44
+tests[116]='my @a=(1,2);print $a[0],"\n";'
+result[116]='1'
+# issue45
+tests[117]='my $x;$x//=1;print "ok" if $x;'
+result[117]='ok'
+# issue46
+tests[118]='my $pattern="x";"foo"=~/$pattern/o;print "ok";'
+result[118]='ok'
+# issue47
+tests[119]='my $f=sub{while(1){return(1);}};print $f->(),"\n";'
+result[119]='1'
+# issue48
+tests[120]='sub f{()}print((my ($v)=f())?1:2,"\n");'
+result[120]='2'
+# issue49
+tests[121]='while(1){while(1){last;}last;}print "ok"'
+result[121]='ok'
+# issue51
+tests[122]='my ($p1,$p2)=(80,80);if($p1<=23&&23<=$p2){print "telnet\n";}elsif ($p1 <= 80 && 80 <= $p2){print "http\n";}else{print "fail\n"}'
+result[122]='http'
+# issue52
+tests[123]='my $x;my $y = 1;$x and $y == 2;print $y == 1 ? "ok\n" : "fail\n";'
+result[123]='ok'
 
 init
 
 # 
-# getopts for -q -k -E -Du,-q -v -O2, -a -c
-while getopts "hqackoED:B:O:" opt
+# getopts for -q -k -E -Du,-q -v -O2, -a -c -fro-inc
+while getopts "hackoED:B:O:f:q" opt
 do
   if [ "$opt" = "q" ]; then 
     QUIET=1
+    CCMD="$CCMD -q"
+  fi
+  if [ "$opt" = "o" ]; then Mblib=" "; init; fi
+  if [ "$opt" = "c" ]; then CONT=1; fi
+  if [ "$opt" = "k" ]; then KEEP=1; fi
+  if [ "$opt" = "E" ]; then CPP=1; fi
+  if [ "$opt" = "h" ]; then help; exit; fi
+  # -D options: u,-q for quiet, no -D for verbose, -D- for no gcc warnings
+  if [ "$opt" = "D" ]; then
+    OCMD="$PERL $Mblib -MO=C,-D${OPTARG},"
+    if [ $BASE = "testcc.sh" ]; then 
+        OCMD="$PERL $Mblib -MO=CC,-D${OPTARG},"
+    fi
+    if [ -z "${OPTARG/-/}" ]; then
+        CCMD="$CCMD -d"
+    fi
+  fi
+  # -B dynamic or -B static
+  if [ "$opt" = "B" ]; then 
+    CCMD="$CCMD -B${OPTARG}"
+  fi
+  if [ "$opt" = "O" ]; then OPTIM="$OPTARG"; fi
+  if [ "$opt" = "f" ]; then
+    OCMD="$(echo $OCMD|sed -e "s/C,/C,-f$OPTARG,/")"
+  fi
+  if [ "$opt" = "a" ]; then # replace -Du, by -Do
+    OCMD="$(echo $OCMD|sed -r -e 's/(-D.*)u,/\1o,/')" 
+  fi
+done
+
+test "$(perl -V:gccversion)" = "gccversion='';" || CCMD="$CCMD -g3"
+if [ -z $OPTIM ]; then OPTIM=-1; fi # all
+
+if [ -z "$QUIET" ]; then
+    make 
+else
     # O from 5.6 does not support -qq
     qq="`$PERL -e'print (($] < 5.007) ? q() : q(-qq,))'`"
     # replace -D*,-v by -q 
@@ -323,35 +421,8 @@ do
     OCMDO2="$(echo $OCMDO2|sed -e 's/-D.*,//' -e 's/,-v,/,/' -e s/-MO=/-MO=$qq/)"
     OCMDO3="$(echo $OCMDO3|sed -e 's/-D.*,//' -e 's/,-v,/,/' -e s/-MO=/-MO=$qq/)"
     OCMDO4="$(echo $OCMDO4|sed -e 's/-D.*,//' -e 's/,-v,/,/' -e s/-MO=/-MO=$qq/)"
-    CCMD="$PERL $Mblib script/cc_harness -q -g3 -DALLOW_PERL_OPTIONS"
-  fi
-  if [ "$opt" = "o" ]; then Mblib=" "; init; fi
-  if [ "$opt" = "c" ]; then CONT=1; fi
-  if [ "$opt" = "k" ]; then KEEP=1; fi
-  if [ "$opt" = "E" ]; then CPP=1; fi
-  if [ "$opt" = "h" ]; then help; exit; fi
-  # -D options: u,-q for quiet, no -D for verbose
-  if [ "$opt" = "D" ]; then
-    OCMD="$PERL $Mblib -MO=C,-D${OPTARG},"
-    if [ $BASE = "testcc.sh" ]; then 
-        OCMD="$PERL $Mblib -MO=CC,-D${OPTARG},"
-    fi
-  fi
-  # -B dynamic or -B static
-  if [ "$opt" = "B" ]; then 
-    CCMD="$PERL $Mblib script/cc_harness -d -g3 -B${OPTARG} -DALLOW_PERL_OPTIONS"
-  fi
-  if [ "$opt" = "O" ]; then OPTIM="$OPTARG"; fi
-  if [ "$opt" = "a" ]; then # replace -Du, by -Do
-    OCMD="$(echo $OCMD|sed -r -e 's/(-D.*)u,/\1o,/')" 
-  fi
-done
-if [ -z $OPTIM ]; then OPTIM=-1; fi # all
-
-if [ -z "$QUIET" ]; then
-    make 
-else
-    make --silent >/dev/null
+    # gnu make?
+    make --silent >/dev/null || make 2&>1 >/dev/null
 fi
 
 # need to shift the options
@@ -366,7 +437,7 @@ else
   for b in $(seq -f"%02.0f" $ntests); do
     ctest $b
   done
-  if [ $BASE = "testcc.sh" ]; then 
+  if [ $BASE = "testcc.sh" ]; then
     for b in $(seq -f"%02.0f" 101 $(($ncctests+100))); do
       ctest $b
     done
